-- Format:		Maxscript 3ds Max 4
--
-- Title:		Standard Mesh Importer
--
-- Purpose: Import a standard mesh (.sm) from Battlefield 1942
--
-- Author:			Rex Hill
-- Creation Date:	Aug 4th 2002
-- Current Version:	1.0 alpha
-- Last Update:		Aug/4
--

clearlistener()
SMscale = .1 -- not used in importing
DSMscale = 10

f = 1 -- just for closing later

objectName = "testBF_m1"
fname = getOpenFileName caption:"Import .sm" types:"Standard Mesh .sm|*.sm|All|*.*|"
if fname != undefined do
(
	print "Reading .sm....."

	f=fopen fname "rb"
	SM_HeaderV = (Readlong f)
	if SM_HeaderV <= 10 do
	(
		qVerts = #()
		VertMod = #()

		qFaces = #()
		FaceMod = #()

	--	normalFace = #()
	--	q2Verts = #()

		fseek f 4 #seek_cur

		-- Bounding Box
		extractBB = false
		if extractBB == true then
		(
			tmp1 = (ReadFloat f)*DSMscale
			tmp2 = (ReadFloat f)*DSMscale
			tmp3 = (ReadFloat f)*DSMscale
			tmp4 = (ReadFloat f)*DSMscale
			tmp5 = (ReadFloat f)*DSMscale
			tmp6 = (ReadFloat f)*DSMscale
			boundingBox = box length:(tmp6-tmp3) width:(tmp4-tmp1) height:(tmp5-tmp2)
			boundingBox.name = (objectName+"_bb")
			boundingBox.pos = [((tmp4-tmp1)*0.5+tmp1) , ((tmp6-tmp3)*0.5+tmp3) , tmp2]
			convertTo boundingBox TriMeshGeometry
		)
		else
		(
			fseek f 24 #seek_cur
		)


		if SM_HeaderV == 10 do
		(
			fseek f 1 #seek_cur
		)

		numTopModels = (Readlong f #unsigned)
		for i=1 to numTopModels do
		(
			SizeOfSection = (Readlong f #unsigned)
			endOffset = (ftell f)+SizeOfSection
		
		
				-- these 8 bytes seem to be a header
			fseek f 8 #seek_cur
				
			numVerts = (Readlong f #unsigned)
			for j=1 to numVerts do
			(	
				vert1 = (ReadFloat f)*DSMscale
				vert2 = (ReadFloat f)*DSMscale
				vert3 = (ReadFloat f)*DSMscale
				VertMod[j] = (ReadShort f #unsigned)+1 -- MatID?
				fseek f 2 #seek_cur -- Stuff of vert1 Repeated
				qVerts[j] = (point3 vert1 vert3 vert2)
			)

			numFaces = (Readlong f #unsigned)
			for j=1 to numFaces do
			(	
				FaceA = (ReadShort f #unsigned)+1
				FaceB = (ReadShort f #unsigned)+1 
				FaceC = (ReadShort f #unsigned)+1
				FaceMod[j] = (ReadShort f #unsigned)+1 -- MatID?
				qFaces[j] = (point3 FaceA FaceB FaceC)
			)

			Unknown1 = ReadLong f #unsigned
			Unknown2 = ReadLong f #unsigned
			FaceNormals = ReadLong f #unsigned

			CollisionMesh = mesh Vertices:qVerts faces:qFaces materialIDS:FaceMod
			CollisionMesh.name = (objectName+"_c0"+(i as string))

			for j=1 to FaceNormals do
			(	
				vert1 = (ReadFloat f)
				vert2 = (ReadFloat f)
				vert3 = (ReadFloat f)
				setFaceNormal CollisionMesh j [vert1,vert3,vert2]
				fseek f 20 #seek_cur
			)
			

				tmp1 = ftell f
				print ("EOS "+(i as string)+": "+(tmp1 as string))
				print ("EOS "+"END: "+(endOffset as string))
				print ("EOS Size "+(i as string)+": "+((endOffset-tmp1) as string))
				fseek f endOffset #seek_set


			



			print ("FaceMod: "+(FaceMod as string))
			print ("Unknown1: "+(Unknown1 as string))
			print ("Unknown2: "+(Unknown2 as string))
			print ("FaceNormals: "+(FaceNormals as string))
			update CollisionMesh

		)
		
	)

	



	fclose f
)
